'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var store = require('svelte/store');

var OBSERVABLE;

function isObservable(value) {
  // Lazy-load Symbol to give polyfills a chance to run
  if (!OBSERVABLE) {
    OBSERVABLE =
      (typeof Symbol === 'function' && Symbol.observable) || '@@observable';
  }

  return value && value[OBSERVABLE] && value[OBSERVABLE]() === value;
}

function pending() {
  return new Promise(noop);
}

function fulfilled(value) {
  return Promise.resolve(value);
}

function rejected(error) {
  return Promise.reject(error);
}

function noop() {}

function observe(observable) {
  if (!isObservable(observable)) {
    return store.readable(noop, observable);
  }

  return store.readable(function (set) {
    var subscription = observable.subscribe({
      next: function next(value) {
        set(fulfilled(value));
      },
      error: function error(err) {
        set(rejected(err));
      }
    });

    return function () { return subscription.unsubscribe(); };
  }, pending());
}

function flat(subscribable) {
  var is_observable = isObservable(subscribable);
  var initial = is_observable ? pending() : undefined;

  return store.readable(function (set) {
    var inner_unsubscribe;
    var outer_unsubscribe;

    var fulfill = is_observable
      ? function (value) { return set(fulfilled(value)); }
      : function (value) { return set(value); };

    function next(value) {
      if (inner_unsubscribe) {
        inner_unsubscribe();
        inner_unsubscribe = null;
      }
      if (isObservable(value)) { value = observe(value); }

      if (isStore(value)) {
        inner_unsubscribe = value.subscribe(function (inner) { return fulfill(inner); });
      } else {
        fulfill(value);
      }
    }
    function error(err) {
      set(rejected(err));
    }

    if (is_observable) {
      var subscription = subscribable.subscribe({ next: next, error: error });
      outer_unsubscribe = function () { return subscription.unsubscribe(); };
    } else {
      outer_unsubscribe = subscribable.subscribe(next);
    }

    return function () {
      if (inner_unsubscribe) { inner_unsubscribe(); }
      outer_unsubscribe();
    };
  }, initial);
}

function isStore(value) {
  return value && typeof value.subscribe === 'function';
}

exports.observe = observe;
exports.flat = flat;
//# sourceMappingURL=svelte-observable.cjs.js.map
